@@ -8,16 +8,10 @@ typedef struct unix_setup_struct {
 	int random_seed;
 	int tandy_bit;
 	int current_text_style;		/* also in ux_text.c and ux_screen.c */
-        int curses_active;
-        int plain_ascii;
+	int curses_active;
+	int plain_ascii;
 	int current_color;		/* ux_text.c ux_screen.c */
 	bool color_enabled;		/* ux_init.c ux_pic.c ux_text.c */
-
-	char *blorb_name;
-	char *blorb_file;
-	bool use_blorb;
-	bool exec_in_blorb;
-
 	int interpreter;		/* see frotz.h */
 } u_setup_t;
 
